@@ -1,17 +1,23 @@
 #ifndef _I3STATUS_H
 #define _I3STATUS_H
 
-enum { O_DZEN2,
-       O_XMOBAR,
-       O_I3BAR,
-       O_LEMONBAR,
-       O_TERM,
-       O_NONE } output_format;
+typedef enum {
+    O_DZEN2,
+    O_XMOBAR,
+    O_I3BAR,
+    O_LEMONBAR,
+    O_TERM,
+    O_NONE
+} output_format_t;
+extern output_format_t output_format;
 
-enum { M_PANGO,
-       M_NONE } markup_format;
+typedef enum {
+    M_PANGO,
+    M_NONE
+} markup_format_t;
+extern markup_format_t markup_format;
 
-char *pct_mark;
+extern char *pct_mark;
 
 #include <stdbool.h>
 #include <confuse.h>
@@ -100,32 +106,40 @@ char *pct_mark;
         }                                                                              \
     } while (0)
 
-#define SEC_CLOSE_MAP                                                                                    \
-    do {                                                                                                 \
-        if (output_format == O_I3BAR) {                                                                  \
-            char *_align = cfg_getstr(sec, "align");                                                     \
-            if (_align) {                                                                                \
-                yajl_gen_string(json_gen, (const unsigned char *) "align", strlen("align"));             \
-                yajl_gen_string(json_gen, (const unsigned char *)_align, strlen(_align));                \
-            }                                                                                            \
-            struct min_width *_width = cfg_getptr(sec, "min_width");                                     \
-            if (_width) {                                                                                \
-                /* if the value can be parsed as a number, we use the numerical value */                 \
-                if (_width->num > 0) {                                                                   \
-                    yajl_gen_string(json_gen, (const unsigned char *) "min_width", strlen("min_width")); \
-                    yajl_gen_integer(json_gen, _width->num);                                             \
-                } else {                                                                                 \
-                    yajl_gen_string(json_gen, (const unsigned char *) "min_width", strlen("min_width")); \
-                    yajl_gen_string(json_gen, (const unsigned char *)_width->str, strlen(_width->str));  \
-                }                                                                                        \
-            }                                                                                            \
-            const char *_sep = cfg_getstr(cfg_general, "separator");                                     \
-            if (strlen(_sep) == 0) {                                                                     \
-                yajl_gen_string(json_gen, (const unsigned char *) "separator", strlen("separator"));     \
-                yajl_gen_bool(json_gen, false);                                                          \
-            }                                                                                            \
-            yajl_gen_map_close(json_gen);                                                                \
-        }                                                                                                \
+#define SEC_CLOSE_MAP                                                                                                        \
+    do {                                                                                                                     \
+        if (output_format == O_I3BAR) {                                                                                      \
+            char *_align = cfg_getstr(sec, "align");                                                                         \
+            if (_align) {                                                                                                    \
+                yajl_gen_string(json_gen, (const unsigned char *) "align", strlen("align"));                                 \
+                yajl_gen_string(json_gen, (const unsigned char *)_align, strlen(_align));                                    \
+            }                                                                                                                \
+            struct min_width *_width = cfg_getptr(sec, "min_width");                                                         \
+            if (_width) {                                                                                                    \
+                /* if the value can be parsed as a number, we use the numerical value */                                     \
+                if (_width->num > 0) {                                                                                       \
+                    yajl_gen_string(json_gen, (const unsigned char *) "min_width", strlen("min_width"));                     \
+                    yajl_gen_integer(json_gen, _width->num);                                                                 \
+                } else {                                                                                                     \
+                    yajl_gen_string(json_gen, (const unsigned char *) "min_width", strlen("min_width"));                     \
+                    yajl_gen_string(json_gen, (const unsigned char *)_width->str, strlen(_width->str));                      \
+                }                                                                                                            \
+            }                                                                                                                \
+            if (cfg_size(sec, "separator") > 0) {                                                                            \
+                yajl_gen_string(json_gen, (const unsigned char *) "separator", strlen("separator"));                         \
+                yajl_gen_bool(json_gen, cfg_getbool(sec, "separator"));                                                      \
+            }                                                                                                                \
+            if (cfg_size(sec, "separator_block_width") > 0) {                                                                \
+                yajl_gen_string(json_gen, (const unsigned char *) "separator_block_width", strlen("separator_block_width")); \
+                yajl_gen_integer(json_gen, cfg_getint(sec, "separator_block_width"));                                        \
+            }                                                                                                                \
+            const char *_sep = cfg_getstr(cfg_general, "separator");                                                         \
+            if (strlen(_sep) == 0) {                                                                                         \
+                yajl_gen_string(json_gen, (const unsigned char *) "separator", strlen("separator"));                         \
+                yajl_gen_bool(json_gen, false);                                                                              \
+            }                                                                                                                \
+            yajl_gen_map_close(json_gen);                                                                                    \
+        }                                                                                                                    \
     } while (0)
 
 #define START_COLOR(colorstr)                                                                \
@@ -160,10 +174,6 @@ char *pct_mark;
         }                                                                                      \
     } while (0)
 
-typedef enum { CS_DISCHARGING,
-               CS_CHARGING,
-               CS_FULL } charging_status_t;
-
 /*
  * The "min_width" module option may either be defined as a string or a number.
  */
@@ -172,6 +182,8 @@ struct min_width {
     const char *str;
 };
 
+char *sstrdup(const char *str);
+
 /* src/general.c */
 char *skip_character(char *input, char character, int amount);
 void die(const char *fmt, ...);
@@ -193,26 +205,27 @@ void set_timezone(const char *tz);
 /* src/first_network_device.c */
 typedef enum {
     NET_TYPE_WIRELESS = 0,
-    NET_TYPE_ETHERNET = 1
+    NET_TYPE_ETHERNET = 1,
+    NET_TYPE_OTHER = 2
 } net_type_t;
 const char *first_eth_interface(const net_type_t type);
 
 void print_ipv6_info(yajl_gen json_gen, char *buffer, const char *format_up, const char *format_down);
-void print_disk_info(yajl_gen json_gen, char *buffer, const char *path, const char *format, const char *format_not_mounted, const char *prefix_type, const char *threshold_type, const double low_threshold);
-void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char *path, const char *format, const char *format_down, const char *status_chr, const char *status_bat, const char *status_full, int low_threshold, char *threshold_type, bool last_full_capacity, bool integer_battery_capacity, bool hide_seconds);
-void print_time(yajl_gen json_gen, char *buffer, const char *title, const char *format, const char *tz, const char *format_time, time_t t);
+void print_disk_info(yajl_gen json_gen, char *buffer, const char *path, const char *format, const char *format_below_threshold, const char *format_not_mounted, const char *prefix_type, const char *threshold_type, const double low_threshold);
+void print_battery_info(yajl_gen json_gen, char *buffer, int number, const char *path, const char *format, const char *format_down, const char *status_chr, const char *status_bat, const char *status_unk, const char *status_full, int low_threshold, char *threshold_type, bool last_full_capacity, bool integer_battery_capacity, bool hide_seconds);
+void print_time(yajl_gen json_gen, char *buffer, const char *title, const char *format, const char *tz, const char *locale, const char *format_time, time_t t);
 void print_ddate(yajl_gen json_gen, char *buffer, const char *format, time_t t);
 const char *get_ip_addr(const char *interface);
 void print_wireless_info(yajl_gen json_gen, char *buffer, const char *interface, const char *format_up, const char *format_down);
 void print_run_watch(yajl_gen json_gen, char *buffer, const char *title, const char *pidfile, const char *format, const char *format_down);
 void print_path_exists(yajl_gen json_gen, char *buffer, const char *title, const char *path, const char *format, const char *format_down);
-void print_cpu_temperature_info(yajl_gen json_gen, char *buffer, int zone, const char *path, const char *format, int);
-void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format);
+void print_cpu_temperature_info(yajl_gen json_gen, char *buffer, int zone, const char *path, const char *format, const char *format_above_threshold, int);
+void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format, const char *format_above_threshold, const char *format_above_degraded_threshold, const float max_threshold, const float degraded_threshold);
 void print_eth_info(yajl_gen json_gen, char *buffer, const char *interface, const char *format_up, const char *format_down);
-void print_load(yajl_gen json_gen, char *buffer, const char *format, const float max_threshold);
+void print_load(yajl_gen json_gen, char *buffer, const char *format, const char *format_above_threshold, const float max_threshold);
 void print_volume(yajl_gen json_gen, char *buffer, const char *fmt, const char *fmt_muted, const char *device, const char *mixer, int mixer_idx);
 bool process_runs(const char *path);
-int volume_pulseaudio(uint32_t sink_idx);
+int volume_pulseaudio(uint32_t sink_idx, const char *sink_name);
 bool pulse_initialize(void);
 
 /* socket file descriptor for general purposes */
@@ -222,7 +235,5 @@ extern cfg_t *cfg, *cfg_general, *cfg_section;
 
 extern void **cur_instance;
 
-extern pthread_cond_t i3status_sleep_cond;
-extern pthread_mutex_t i3status_sleep_mutex;
-
+extern pthread_t main_thread;
 #endif
