@@ -41,11 +41,13 @@ static int prev_idle = 0;
  * percentage.
  *
  */
-void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format) {
+void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format, const char *format_above_threshold, const char *format_above_degraded_threshold, const float max_threshold, const float degraded_threshold) {
+    const char *selected_format = format;
     const char *walk;
     char *outwalk = buffer;
     int curr_user = 0, curr_nice = 0, curr_system = 0, curr_idle = 0, curr_total;
     int diff_idle, diff_total, diff_usage;
+    bool colorful_output = false;
 
 #if defined(LINUX)
     static char statpath[512];
@@ -95,7 +97,20 @@ void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format) {
 #else
     goto error;
 #endif
-    for (walk = format; *walk != '\0'; walk++) {
+
+    if (diff_usage >= max_threshold) {
+        START_COLOR("color_bad");
+        colorful_output = true;
+        if (format_above_threshold != NULL)
+            selected_format = format_above_threshold;
+    } else if (diff_usage >= degraded_threshold) {
+        START_COLOR("color_degraded");
+        colorful_output = true;
+        if (format_above_degraded_threshold != NULL)
+            selected_format = format_above_degraded_threshold;
+    }
+
+    for (walk = selected_format; *walk != '\0'; walk++) {
         if (*walk != '%') {
             *(outwalk++) = *walk;
             continue;
@@ -107,6 +122,9 @@ void print_cpu_usage(yajl_gen json_gen, char *buffer, const char *format) {
         }
     }
 
+    if (colorful_output)
+        END_COLOR;
+
     OUTPUT_FULL_TEXT(buffer);
     return;
 error:
